<?php

namespace InspireLabs\WooInpost\shipping;

use Exception;
use InspireLabs\WooInpost\EasyPack;
use InspireLabs\WooInpost\EasyPack_API;
use InspireLabs\WooInpost\shipx\models\shipment\ShipX_Shipment_Model;
use InspireLabs\WooInpost\shipx\models\shipment\ShipX_Shipment_Parcel_Model;
use InspireLabs\WooInpost\shipx\services\shipment\ShipX_Shipment_Service;
use phpDocumentor\Reflection\DocBlock\Tags\Var_;
use ReflectionException;
use WC_Shipping_Method;

/**
 * EasyPack Shipping Method Parcel Machines
 *
 * @author      WPDesk
 * @category    Admin
 * @package     EasyPack/Admin
 * @version     2.1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

if ( ! class_exists( 'EasyPack_Shippng_Parcel_Machines' ) ) {
	class EasyPack_Shippng_Parcel_Machines extends WC_Shipping_Method {

		static $logo_printed;

		static $setup_hooks_once = false;

		const SERVICE_ID = ShipX_Shipment_Model::SERVICE_INPOST_LOCKER_STANDARD;

		const NONCE_ACTION = self::SERVICE_ID;

		static $prevent_duplicate = [];

		static $review_order_after_shipping_once = false;

		/**
		 * Constructor for shipping class
		 *
		 * @access public
		 * @return void
		 */
		public function __construct( $instance_id = 0 ) {
			$this->init_form_fields();

			$this->instance_id = absint( $instance_id );
			$this->supports    = [
				'shipping-zones',
				'instance-settings',
			];

			$this->id = 'easypack_parcel_machines';
			$this->method_description
			          = __( 'Allow customers to pick up orders themselves. By default, when using local pickup store base taxes will apply regardless of customer address.',
				'woocommerce' );

			$this->method_title = __( 'InPost Locker 24/7',
				EasyPack::$text_domain );
			$this->init();

		}


		/**
		 * Init your settings
		 *
		 *
		 * @access public
		 * @return void
		 */
		function init() {


			$this->init_form_fields(); // This is part of the settings API. Override the method to add your own settings
			$this->init_settings(); // This is part of the settings API. Loads settings you previously init.
			$this->title          = $this->get_option( 'title' );
			$this->free_shipping_cost
			                      = $this->get_option( 'free_shipping_cost' );
			$this->flat_rate      = $this->get_option( 'flat_rate' );
			$this->cost_per_order = $this->get_option( 'cost_per_order' );
			$this->based_on       = $this->get_option( 'based_on' );

			$this->tax_status = $this->get_option( 'tax_status' );

			$this->setup_hooks_once();
		}

		private function setup_hooks_once() {

			add_action( 'woocommerce_update_options_shipping_' . $this->id,
				[ $this, 'process_admin_options' ] );

			add_action( 'woocommerce_review_order_after_shipping',
				[ $this, 'woocommerce_review_order_after_shipping' ] );

			add_action( 'woocommerce_checkout_update_order_meta',
				[ $this, 'woocommerce_checkout_update_order_meta' ] );

			add_action( 'woocommerce_checkout_process',
				[ $this, 'woocommerce_checkout_process' ] );

			add_action( 'save_post', [ $this, 'save_post' ] );

			add_action( 'add_meta_boxes', [ $this, 'add_meta_boxes' ], 10, 2 );

			add_filter( 'woocommerce_cart_shipping_method_full_label',
				[ $this, 'woocommerce_cart_shipping_method_full_label' ], 10,
				2 );

			add_filter( 'woocommerce_order_shipping_to_display_shipped_via',
				[
					$this,
					'woocommerce_order_shipping_to_display_shipped_via',
				], 10, 2 );

			add_filter( 'woocommerce_my_account_my_orders_actions',
				[ $this, 'woocommerce_my_account_my_orders_actions' ], 10,
				2 );

			add_action( 'woocommerce_email_after_order_table',
				[ $this, 'woocommerce_email_after_order_table' ], 10, 3 );

		}

		public function admin_options() {
			?>
            <table class="form-table">
				<?php $this->generate_settings_html(); ?>
            </table>
            <script type="text/javascript">
                function display_rates() {
                    if (jQuery('.easypack_flat_rate').prop('checked')) {
                        jQuery('.easypack_cost_per_order').closest('tr').css('display', 'table-row');
                        jQuery('.easypack_based_on').closest('tr').css('display', 'none');
                        jQuery('.easypack_rates').closest('tr').css('display', 'none');
                        jQuery('#woocommerce_easypack_parcel_machines_1').css('display', 'none');
                        jQuery('#woocommerce_easypack_parcel_machines_cod_1').css('display', 'none');
                    } else {
                        jQuery('.easypack_cost_per_order').closest('tr').css('display', 'none');
                        jQuery('.easypack_based_on').closest('tr').css('display', 'table-row');
                        jQuery('.easypack_rates').closest('tr').css('display', 'table-row');
                        jQuery('#woocommerce_easypack_parcel_machines_1').css('display', 'block');
                        jQuery('#woocommerce_easypack_parcel_machines_cod_1').css('display', 'block');
                    }
                }

                jQuery('.easypack_flat_rate').change(function () {
                    display_rates();
                });
                display_rates();
            </script>
			<?php
		}

		public function generate_rates_html( $key, $data ) {
			$rates = get_option( 'woocommerce_' . $this->id . '_rates', [] );
			ob_start();
			include( 'views/html-rates.php' );

			return ob_get_clean();
		}


		public function init_form_fields() {
			$settings                   = [
				[
					'title'       => __( 'General settings',
						EasyPack::$text_domain ),
					'type'        => 'title',
					'description' => '',
					'id'          => 'section_general_settings',
				],
				'logo_upload'        => [
					'name'  => __( 'Change logo', '' ),
					'title' => __( 'Upload custom logo',
						EasyPack::$text_domain ),
					'type'  => 'logo_upload',
					'id'    => 'logo_upload',
				],
				'title'              => [
					'title'    => __( 'Method title', EasyPack::$text_domain ),
					'type'     => 'text',
					'default'  => __( 'InPost Locker 24/7',
						EasyPack::$text_domain ),
					'desc_tip' => false,
				],
				'free_shipping_cost' => [
					'title'             => __( 'Free shipping',
						EasyPack::$text_domain ),
					'type'              => 'number',
					'custom_attributes' => [
						'step' => 'any',
						'min'  => '0',
					],
					'default'           => '',
					'desc_tip'          => __( 'Enter the amount of the order from which the shipping will be free (does not include virtual products). ',
						EasyPack::$text_domain ),
					'placeholder'       => '0.00',
				],
				'flat_rate'          => [
					'title'   => __( 'Flat rate', EasyPack::$text_domain ),
					'type'    => 'checkbox',
					'label'   => __( 'Set a flat-rate shipping fee for the entire order.',
						EasyPack::$text_domain ),
					'class'   => 'easypack_flat_rate',
					'default' => 'yes',
				],
				'cost_per_order'     => [
					'title'             => __( 'Cost per order',
						EasyPack::$text_domain ),
					'type'              => 'number',
					'custom_attributes' => [
						'step' => 'any',
						'min'  => '0',
					],
					'class'             => 'easypack_cost_per_order',
					'default'           => '',
					'desc_tip'          => __( 'Set a flat-rate shipping for all orders'
						, EasyPack::$text_domain ),
					'placeholder'       => '0.00',
				],
				'tax_status' => [
					'title'   => __( 'Tax status', 'woocommerce' ),
					'type'    => 'select',
					'class'   => 'wc-enhanced-select',
					'default' => 'none',
					'options' => [
						'none'    => _x( 'None', 'Tax status',
							EasyPack::$text_domain ),
						'taxable' => __( 'Taxable', EasyPack::$text_domain ),
					],
				],

				[
					'title'       => __( 'Rates table',
						EasyPack::$text_domain ),
					'type'        => 'title',
					'description' => '',
					'id'          => 'section_general_settings',
				],
				'based_on'           => [
					'title'    => __( 'Based on', EasyPack::$text_domain ),
					'type'     => 'select',
					'desc_tip' => __( 'Select the method of calculating shipping cost. If the cost of shipping is to be calculated based on the weight of the cart and the products do not have a defined weight, the cost will be calculated incorrectly.',
						EasyPack::$text_domain ),
					'class'    => 'wc-enhanced-select easypack_based_on',
					'options'  => [
						'price'  => __( 'Price', EasyPack::$text_domain ),
						'weight' => __( 'Weight', EasyPack::$text_domain ),
					],
				],
				'rates'              => [
					'title'    => '',
					'type'     => 'rates',
					'class'    => 'easypack_rates',
					'default'  => '',
					'desc_tip' => '',
				],
			];
			$this->instance_form_fields = $settings;
			$this->form_fields          = $settings;


		}


		public function generate_logo_upload_html( $key, $data ) {
			$field_key = $this->get_field_key( $key );

			$defaults = [
				'title'             => 'Upload custom logo',
				'disabled'          => false,
				'class'             => '',
				'css'               => '',
				'placeholder'       => '',
				'type'              => 'text',
				'desc_tip'          => false,
				'description'       => '',
				'custom_attributes' => [],
			];

			$data = wp_parse_args( $data, $defaults );

			ob_start();
			?>
            <tr valign="top">
                <th scope="row" class="titledesc">
                    <label for="<?php echo esc_attr( $field_key ); ?>"><?php echo wp_kses_post( $data['title'] ); ?><?php echo $this->get_tooltip_html( $data ); // WPCS: XSS ok.
						?></label>
                </th>
                <td class="forminp">
                    <fieldset>
                        <legend class="screen-reader-text">
                            <span><?php echo wp_kses_post( $data['title'] ); ?></span>
                        </legend>
                        <button class='woo-inpost-logo-upload-btn'>Upload logo
                        </button>
                        <img src='<?php echo esc_attr( $this->get_instance_option( $key ) ); ?>'
                             style='width: 60px; height: 60px; background-size: cover'
                             id='woo-inpost-logo-preview'>
                        <input class="input-text regular-input" type="hidden"
                               name="<?php echo esc_attr( $field_key ); ?>"
                               id="woocommerce_easypack_logo_upload"
                               style="<?php echo esc_attr( $data['css'] ); ?>"
                               value="<?php echo esc_attr( $this->get_instance_option( $key ) ); ?>"
                               placeholder="<?php echo esc_attr( $data['placeholder'] ); ?>"/>
						<?php echo $this->get_description_html( $data ); // WPCS: XSS ok.
						?>
                    </fieldset>
                </td>
            </tr>
			<?php

			return ob_get_clean();
		}


		public function add_rate( $args = [] ) {


			$args['meta_data']
				= [ 'logo' => $this->get_instance_option( 'logo_upload' ) ];


			parent::add_rate( $args );
		}

		public function process_admin_options() {
			parent::process_admin_options();

			if ( isset( $_POST['rates'] ) ) {
				$rates = $_POST['rates'];
				update_option( 'woocommerce_' . $this->id . '_rates', $rates );
			}
		}

		public function calculate_shipping_free_shipping( $package ) {
			if ( ! empty( $this->free_shipping_cost )
			     && $this->free_shipping_cost <= $package['contents_cost']
			) {
				$add_rate = [
					'id'    => $this->id,
					'label' => $this->title,
					'cost'  => 0,
				];
				$this->add_rate( $add_rate );

				return true;
			}

			return false;
		}

		public function calculate_shipping_flat( $package ) {
			if ( $this->flat_rate == 'yes' ) {
				$add_rate = [
					'id'    => $this->id,
					'label' => $this->title,
					'cost'  => $this->cost_per_order,
				];
				$this->add_rate( $add_rate );

				return true;
			}

			return false;
		}

		public function package_weight( $items ) {
			$weight = 0;
			foreach ( $items as $item ) {
				$weight += $item['data']->weight * $item['quantity'];
			}

			return $weight;
		}

		public function package_subtotal( $items ) {
			$subtotal = 0;
			foreach ( $items as $item ) {
				$subtotal += $item['line_subtotal']
				             + $item['line_subtotal_tax'];
			}

			return $subtotal;
		}

		/**
		 * @param unknown $package
		 *
		 */
		public function calculate_shipping_table_rate( $package ) {
			$rates = get_option( 'woocommerce_' . $this->id . '_rates', [] );
			foreach ( $rates as $key => $rate ) {
				if ( empty( $rates[ $key ]['min'] )
				     || trim( $rates[ $key ]['min'] ) == ''
				) {
					$rates[ $key ]['min'] = 0;
				}
				if ( empty( $rates[ $key ]['max'] )
				     || trim( $rates[ $key ]['max'] ) == ''
				) {
					$rates[ $key ]['max'] = PHP_INT_MAX;
				}
			}
			$value = 0;
			if ( $this->based_on == 'price' ) {
				$value = $this->package_subtotal( $package['contents'] );
			}
			if ( $this->based_on == 'weight' ) {
				$value = $this->package_weight( $package['contents'] );
			}
			foreach ( $rates as $rate ) {
				if ( floatval( $rate['min'] ) <= $value
				     && floatval( $rate['max'] ) >= $value
				) {
					$add_rate = [
						'id'    => $this->id,
						'label' => $this->title,
						'cost'  => $rate['cost'],
					];
					$this->add_rate( $add_rate );

					return;
				}
			}
		}

		/**
		 * @param array $package
		 */
		public function calculate_shipping( $package = [] ) {
			if ( EasyPack_API()->normalize_country_code_for_inpost(
					$package['destination']['country'] )
			     == EasyPack_API()->getCountry()
			) {

				if ( ! $this->calculate_shipping_free_shipping( $package ) ) {
					if ( ! $this->calculate_shipping_flat( $package ) ) {
						$this->calculate_shipping_table_rate( $package );
					}
				}
			}
		}


		/**
		 *
		 */
		public function woocommerce_review_order_after_shipping() {
			if ( in_array( $this->id,
					WC()->session->get( 'chosen_shipping_methods' ) )
			     && ! self::$review_order_after_shipping_once ) {
				$parcel_machines = EasyPack_API()->machines_options();

				$args = [ 'parcel_machines' => $parcel_machines ];
				$args['parcel_machine_id']
				      = WC()->session->get( 'parcel_machine_id' );
				$args['geowidget_src']
				      = EasyPack_Helper()->get_geowidget_src();
				wc_get_template(
					'checkout/easypack-review-order-after-shipping.php',
					$args,
					'',
					EasyPack()->getTemplatesFullPath()
				);

				self::$review_order_after_shipping_once = true;
			}
		}

		public function woocommerce_checkout_process() {

			if ( in_array( $this->id,
				WC()->session->get( 'chosen_shipping_methods' ) )
			) {
				if ( false === $this->is_method_courier() ) {
					if ( empty( $_POST['parcel_machine_id'] ) ) {
						wc_add_notice( __( 'Parcel locker must be choosen.',
							EasyPack::$text_domain ), 'error' );
					} else {
						WC()->session->set( 'parcel_machine_id',
							$_POST['parcel_machine_id'] );
					}
				}
				$billing_phone = $_POST['billing_phone'];
				if ( false === EasyPack_API()->is_uk() ) {
					$validate_phone
						= EasyPack_API()->validate_phone( $billing_phone );
					if ( $validate_phone !== true ) {
						wc_add_notice( $validate_phone, 'error' );
					}
				}
			}

		}

		public function woocommerce_checkout_update_order_meta( $order_id ) {
			if ( $_POST['parcel_machine_id'] ) {
				update_post_meta( $order_id, '_parcel_machine_id',
					esc_attr( $_POST['parcel_machine_id'] ) );
			}

			if ( $_POST['parcel_machine_desc'] ) {
				update_post_meta( $order_id, '_parcel_machine_desc',
					esc_attr( $_POST['parcel_machine_desc'] ) );
			}

		}

		public function save_post( $post_id ) {
			// Check if our nonce is set.
			if ( ! isset( $_POST['wp_nonce'] ) ) {
				return;
			}
			// Verify that the nonce is valid.
			if ( ! wp_verify_nonce( $_POST['wp_nonce'],
				self::NONCE_ACTION )
			) {
				return;
			}
			// If this is an autosave, our form has not been submitted, so we don't want to do anything.
			if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
				return;
			}

			$status = get_post_meta( $post_id, '_easypack_status', true );
			if ( $status == '' ) {
				$status = 'new';
			}

			if ( $status == 'new' ) {
				$parcel_machine_id   = $_POST['parcel_machine_id'];
				$parcel_machine_desc = $_POST['parcel_machine_desc'];

				update_post_meta( $post_id, '_parcel_machine_id',
					$parcel_machine_id );
				update_post_meta( $post_id, '_parcel_machine_desc',
					$parcel_machine_desc );


				$parcels         = $_POST['parcel'];
				$easypack_pacels = [];
				foreach ( $parcels as $parcel ) {
					$easypack_pacels[] = [ 'package_size' => $parcel ];
				}
				update_post_meta( $post_id, '_easypack_parcels',
					$easypack_pacels );

				$easypack_send_method = $_POST['send_method'];
				update_post_meta( $post_id, '_easypack_send_method',
					$easypack_send_method );
			}

		}

		public function get_logo() {
			//$custom_logo = $this->//get_option( 'logo_upload' );
			$custom_logo = null;

			if ( empty( $custom_logo ) ) {
				return '<img style="height:22px; float:right;" src="'
				       . untrailingslashit( EasyPack()->getPluginImages()
				                            . 'logo/small/white.png"/>' );
			} else {
				return '<img style="height:22px; float:right;" src="'
				       . untrailingslashit( $custom_logo );
			}

		}

		public function add_meta_boxes( $post_type, $post ) {
			if ( $post->post_type == 'shop_order' ) {
				$order = wc_get_order( $post->ID );
				if ( $order->has_shipping_method( $this->id ) ) {
					add_meta_box( 'easypack_parcel_machines'
						,
						__( 'InPost', EasyPack::$text_domain )
						. $this->get_logo()
						, [ $this, 'order_metabox' ]
						, 'shop_order'
						, 'side'
						, 'default'
					);
				}
			}
		}

		public function order_metabox( $post ) {
			self::order_metabox_content( $post );
		}

		/**
		 * @throws ReflectionException
		 */
		public static function ajax_create_package() {
			$ret = [ 'status' => 'ok' ];

			$shipment_model   = self::ajax_create_shipment_model();
			$order_id         = $shipment_model->getInternalData()
			                                   ->getOrderId();
			$shipment_service = EasyPack::EasyPack()->get_shipment_service();
			$status_service   = EasyPack::EasyPack()
			                            ->get_shipment_status_service();
			$shipment_array
			                  = $shipment_service->shipment_to_array( $shipment_model );

			$label_url = '';


			try {
				update_post_meta( $shipment_model->getInternalData()
				                                 ->getOrderId(),
					'_easypack_parcel_create_args',
					$shipment_array );
				$response
					= EasyPack_API()->customer_parcel_create( $shipment_array );

				$internal_data = $shipment_model->getInternalData();
				$internal_data->setInpostId( $response['id'] );
				$internal_data->setStatus( $response['status'] );
				$internal_data->setStatusTitle(
					$status_service->getStatusTitle( $response['status'] )
				);
				$internal_data->setStatusDescription(
					$status_service->getStatusDescription( $response['status'] )
				);

				$internal_data->setStatusChangedTimestamp( time() );
				//$internal_data->setCreatedAt(time());
				$internal_data->setCreatedAt( time() );
				$internal_data->setUrl( $response['href'] );
				$shipment_model->setInternalData( $internal_data );
				$shipment_service->update_shipment_to_db( $shipment_model );

				$search_in_api
					= EasyPack_API()->customer_parcel_get_by_id( $shipment_model->getInternalData()
					                                                            ->getInpostId() );

				$label_url = null;
				//if ('courier' === $_POST['send_method']) {
				for ( $i = 0; $i < 10; $i ++ ) {
					sleep( 1 );
					$label_url
						= self::ajax_parcel_machines_get_stickers_url( $shipment_model );
					if ( ! empty( $label_url ) ) {
						break;
					}
					/*
					 * {"status":400,"error":"invalid_action","message":"Action (get_label) can not be taken on shipment with status (created).","details":{"action":"get_label","shipment_status":"created","shipment_id":53256}}
					 */
				}
				//}

				for ( $i = 0; $i < 3; $i ++ ) {
					sleep( 1 );
					$search_in_api
						= EasyPack_API()->customer_parcel_get_by_id( $shipment_model->getInternalData()
						                                                            ->getInpostId() );
					if ( isset( $search_in_api['items'][0]['tracking_number'] ) ) {
						$shipment_model->getInternalData()->setTrackingNumber(
							$search_in_api['items'][0]['tracking_number']
						);
						break;
					}
				}

				$internal_data = $shipment_model->getInternalData();
				$internal_data->setLabelUrl( $label_url );
				$shipment_model->setInternalData( $internal_data );
				$shipment_service->update_shipment_to_db( $shipment_model );
				update_post_meta( $order_id, '_easypack_status', 'created' );

				//zapisz koszt przesyłki do przesyłki
				$price_calculator
					= EasyPack()->get_shipment_price_calculator_service();

				//$shipment_cost = $price_calculator->get_calculated_shipment_cost($shipment_model);
				//$shipment_model->getInternalData()->setShipmentCost($shipment_cost);

				$shipment_service->update_shipment_to_db( $shipment_model );
			} catch ( Exception $e ) {
				$ret['status'] = 'error';
				$ret['message']
				               = __( 'There are some errors. Please fix it: <br>'
						, EasyPack::$text_domain )
				                 . $e->getMessage();
			}


			if ( $ret['status'] == 'ok' ) {
				wc_get_order( $order_id )->add_order_note(
					__( 'Shipment created', EasyPack::$text_domain ), false
				);
				$ret['content']
					= self::order_metabox_content( get_post( $order_id ), false,
					$shipment_model );
			}
			echo json_encode( $ret );
			wp_die();
		}

		/**
		 * @param                           $post
		 * @param bool $output
		 * @param ShipX_Shipment_Model|null $shipment
		 *
		 * @return string
		 * @throws Exception
		 */
		public static function order_metabox_content(
			$post,
			$output = true,
			$shipment = null
		) {
			$organization_service = EasyPack::EasyPack()
			                                ->get_organization_service();

			$organization = $organization_service->query_organisation();

			if ( ! $organization ) {
				$services_for_organization = [];
			} else {
				$services_for_organization = $organization->getServices();
			}


			if ( ! $output ) {
				ob_start();
			}
			$shipment_service = EasyPack::EasyPack()->get_shipment_service();
			$order_id         = $post->ID;
			if ( false === $shipment instanceof ShipX_Shipment_Model ) {
				$shipment
					= $shipment_service->get_shipment_by_order_id( $order_id );
			}

			if ( $shipment instanceof ShipX_Shipment_Model
			     && false === $shipment_service
					->is_shipment_match_to_current_api( $shipment )
			) {
				wp_nonce_field( self::NONCE_ACTION,
					'wp_nonce' );
				$wrong_api_env = true;
				include( 'views/html-order-matabox-parcel-machines.php' );
				if ( ! $output ) {
					$out = ob_get_clean();

					return $out;
				}

				return '';
			}
			$wrong_api_env = false;

			$order = wc_get_order( $order_id );

			$parcel_machines = EasyPack_API()->machines_options();
			/**
			 * id, template, dimensions, weight, tracking_number, is_not_standard
			 */

			if ( null !== $shipment ) {
				$parcels      = $shipment->getParcels();
				$tracking_url = $shipment->getInternalData()
				                         ->getTrackingNumber();
				$stickers_url = $shipment->getInternalData()->getLabelUrl();

				if ( true === $output ) {
					$status_srv = EasyPack()->get_shipment_status_service();
					$status_srv->refreshStatus( $shipment );
				}

				$status            = $shipment->getInternalData()->getStatus();
				$parcel_machine_id = $shipment->getCustomAttributes()
				                              ->getTargetPoint();
				$send_method       = $shipment->getCustomAttributes()
				                              ->getSendingMethod();
				$disabled          = true;
			} else {
				$package_sizes_display
					     = EasyPack()->get_package_sizes_display();
				$parcels = [];
				$parcel  = new ShipX_Shipment_Parcel_Model();
				$parcel->setTemplate( get_option( 'easypack_default_package_size',
					'small' ) );
				$parcels[] = $parcel;

				$parcel_machine_from_order
					               = $order->get_meta( '_parcel_machine_id',
					$order_id );
				$parcel_machine_id = ! empty( $parcel_machine_from_order )
					? $parcel_machine_from_order
					: get_option( 'easypack_default_machine_id' );

				$tracking_url = false;
				$status       = 'new';
				$send_method  = get_option( 'easypack_default_send_method',
					'parcel_machine' );
				$disabled     = false;
			}
			$package_sizes = EasyPack()->get_package_sizes();

			$send_method_disabled = false;

			if ( EasyPack_API()->getCountry() === EasyPack_API::COUNTRY_PL ) {
				$send_methods = [
					'parcel_machine' => __( 'Parcel locker',
						EasyPack::$text_domain )
					,
					'courier'        => __( 'Courier', EasyPack::$text_domain )
					,
					'pop'            => __( 'POP', EasyPack::$text_domain ),
				];
			} else {
				$send_methods = [
					'parcel_machine' => __( 'Parcel locker',
						EasyPack::$text_domain ),
				];
			}
			$selected_service
				= $shipment_service->get_customer_service_name_by_id(
				self::SERVICE_ID );
			include( 'views/html-order-matabox-parcel-machines.php' );


			wp_nonce_field( self::NONCE_ACTION,
				'wp_nonce' );
			if ( ! $output ) {
				$out = ob_get_clean();

				return $out;
			}
		}


		public static function ajax_parcel_machines_get_stickers() {
			$selected_shipments = $_POST['easypack_parcel'];

			try {
				$labels
					= EasyPack_API()->customer_shipments_labels( $selected_shipments );

				die;


				//echo EasyPack()::LABELS_DIRECTORY . DIRECTORY_SEPARATOR . 'label.pdf';

				$upload     = wp_upload_dir();
				$upload_dir = $upload['basedir'];
				$upload_dir = $upload_dir . DIRECTORY_SEPARATOR
				              . 'woo_inpost_uploads' . DIRECTORY_SEPARATOR;


				if ( ! is_dir( $upload_dir ) ) {
					mkdir( $upload_dir, 0775 );
				}

				$label_name = 'label_' . $order_id . '.pdf';

				file_put_contents(
					$upload_dir . DIRECTORY_SEPARATOR . $label_name
					, $label['body']
				);

				$ret['status'] = 'success';
				$ret['url']    = content_url() . '/uploads/woo_inpost_uploads/'
				                 . $label_name;

				echo json_encode( $ret );
				exit;

			} catch ( Exception $e ) {
				$ret['status'] = 'error';
				$ret['message']
				               = __( 'There are some errors. Please fix it: <br>'
						, EasyPack::$text_domain )
				                 . $e->getMessage();
			}
		}


		/**
		 * @param ShipX_Shipment_Model $shipment_model
		 *
		 * @return string
		 */
		public static function ajax_parcel_machines_get_stickers_url(
			$shipment_model
		) {
			$order_id = $shipment_model->getInternalData()->getOrderId();

			try {
				$label
					        = EasyPack_API()->customer_parcel_sticker( $shipment_model->getInternalData()
					                                                                  ->getInpostId() );
				$upload     = wp_upload_dir();
				$upload_dir = $upload['basedir'];
				$upload_dir = $upload_dir . DIRECTORY_SEPARATOR
				              . 'woo_inpost_uploads' . DIRECTORY_SEPARATOR;


				if ( ! is_dir( $upload_dir ) ) {
					mkdir( $upload_dir, 0775 );
				}

				$label_name = 'label_' . $order_id . '.pdf';

				file_put_contents(
					$upload_dir . DIRECTORY_SEPARATOR . $label_name
					, $label['body']
				);

				return content_url() . '/uploads/woo_inpost_uploads/'
				       . $label_name;

			} catch ( Exception $e ) {
				$ret['status'] = 'error';
				$ret['message']
				               = __( 'There are some errors. Please fix it: <br>'
						, EasyPack::$text_domain )
				                 . $e->getMessage();

				return $ret;
			}
		}


		/**
		 * @return ShipX_Shipment_Model
		 */
		public static function ajax_create_shipment_model() {
			$shipmentService = EasyPack::EasyPack()->get_shipment_service();

			if ( ! isset( $_POST['insurance_amounts'] ) || $_POST['insurance_amounts'][0] === '0' ) {
				$insurance_amounts = [ null ];
			} else {
				$insurance_amounts = $_POST['insurance_amounts'];
			}

			$shipment
				= $shipmentService->create_shipment_object_by_shiping_data(
				$_POST['parcels'],
				(int) $_POST['order_id'],
				$_POST['send_method'],
				self::SERVICE_ID,
				[],
				$_POST['parcel_machine_id'],
				null,
				$insurance_amounts[0],
				$_POST['reference_number']
			);
			$shipment->getInternalData()
			         ->setOrderId( (int) $_POST['order_id'] );

			return $shipment;
		}

		public static function ajax_cancel_package() {

			$ret              = [ 'status' => 'ok', 'message' => '' ];
			$order_id         = $_POST['order_id'];
			$order            = wc_get_order( $order_id );
			$post             = get_post( $order_id );
			$shipment_service = EasyPack::EasyPack()->get_shipment_service();
			$shipment
			                  = $shipment_service->get_shipment_by_order_id( $order_id );

			try {
				$cancelled_parcel
					= EasyPack_API()->customer_parcel_cancel( $shipment->getInternalData()
					                                                   ->getInpostId() );

			} catch ( Exception $e ) {
				$ret['status']  = 'error';
				$ret['message'] .= $e->getMessage();
			}

			$status_srv = EasyPack()->get_shipment_status_service();
			$status_srv->refreshStatus( $shipment );
			if ( $ret['status'] === 'ok' ) {

				$order->add_order_note( __( 'Shipment canceled',
					EasyPack::$text_domain ), false );
				$ret['content'] = self::order_metabox_content( $post, false );
			}
			echo json_encode( $ret );
			wp_die();
		}

		public static function ajax_dispatch_order() {

			$ret = [ 'status' => 'ok', 'message' => '' ];

			$parcels        = $_POST['parcels'];
			$ret['parcels'] = $parcels;

			$dispatch_point        = $_POST['dispatch_point'];
			$ret['dispatch_point'] = $dispatch_point;

			$parcel_ids = [];
			foreach ( $parcels as $parcel ) {
				$parcel_exploded = explode( '.', $parcel );
				$parcel_ids[]    = $parcel_exploded[2];
			}

			$args = [ 'parcel_ids' => $parcel_ids ];

			try {
				$response  = EasyPack_API()->dispatch_order( $dispatch_point,
					$args );
				$ret['id'] = $response['id'];
			} catch ( Exception $e ) {
				$ret['status']  = 'error';
				$ret['message'] .= $e->getMessage();
			}

			echo json_encode( $ret );
			wp_die();

		}

		public static function get_stickers() {

			$nonce = $_GET['security'];
			if ( ! wp_verify_nonce( $nonce, 'easypack_nonce' ) ) {
				echo __( 'Security check - bad nonce!',
					EasyPack::$text_domain );

				return;
			}

			$order_id = $_GET['order_id'];
			$order    = wc_get_order( $order_id );
			$post     = get_post( $order_id );

			$status = get_post_meta( $order_id, '_easypack_status', true );

			$easypack_parcels = get_post_meta( $order_id, '_easypack_parcels',
				true );


			$stickers = [];

			if ( $easypack_parcels ) {
				foreach ( $easypack_parcels as $key => $parcel ) {
					try {
						$easypack_data
							= EasyPack_API()->customer_parcel( $parcel['easypack_data']['id'] );
						if ( $parcel['easypack_data']['status']
						     != $easypack_data['status']
						) {
							$parcel['easypack_data']  = $easypack_data;
							$easypack_parcels[ $key ] = $parcel;
							update_post_meta( $order_id, '_easypack_parcels',
								$easypack_parcels );
						}
						if ( $parcel['easypack_data']['status']
						     == 'created'
						) {
							$easypack_data
								= EasyPack_API()->customer_parcel_pay( $parcel['easypack_data']['id'] );
							$easypack_parcels[ $key ]['easypack_data']
								= $easypack_data;
							update_post_meta( $order_id, '_easypack_parcels',
								$easypack_parcels );
						}
						$stickers[]
							= EasyPack_API()->customer_parcel_sticker( $parcel['easypack_data']['id'] );
					} catch ( Exception $e ) {
						echo $e->getMessage();

						return;
					}
				}
			}

			$file = EasyPack_Helper()->write_stickers_to_file( $stickers );
			if ( $status == 'created' ) {
				update_post_meta( $order_id, '_easypack_status', 'prepared' );
			}
			EasyPack_Helper()->get_file( $file,
				__( 'stickers', EasyPack::$text_domain ) . '_'
				. $order->get_id()
				. '.pdf', 'application/pdf' );
		}

		function woocommerce_cart_shipping_method_full_label(
			$label,
			$method
		) {
			return $label;

			if ( in_array( $this->id, self::$prevent_duplicate ) ) {
				return $label;
			}


			if ( $method->id === $this->id ) {

				/*$custom_logo = $this->get_rate_id('logo');
				$custom_logo = $this->get_instance_option( $custom_logo );
				var_dump($custom_logo);die;*/

				if ( empty( $custom_logo ) ) {
					$img
						   = ' <span class="easypack-shipping-method-logo"><img style="" src="'
						     . EasyPack()->getPluginImages()
						     . 'logo/small/white.png" /><span>';
					$label .= $img;
				} else {
					$img
						   = ' <span class="easypack-shipping-method-logo"><img style="" src="'
						     . $custom_logo . '" /><span>';
					$label .= $img;
				}

				self::$prevent_duplicate[] = $this->id;

			}


			return $label;
		}

		function woocommerce_order_shipping_to_display_shipped_via(
			$via,
			$order
		) {

			if ( self::$logo_printed === 1 ) {
				return $via;
			}

			if ( $order->has_shipping_method( $this->id ) ) {
				$img
					                = ' <span class="easypack-shipping-method-logo" style="display: inline;"><img style="max-width: 100px; max-height: 40px;	display: inline; border:none;" src="'
					                  . EasyPack()->getPluginUrl()
					                  . 'logo/small/white.png" /><span>';
				$via                .= $img;
				self::$logo_printed = 1;
			}

			return $via;
		}

		function woocommerce_my_account_my_orders_actions( $actions, $order ) {
			if ( $order->has_shipping_method( $this->id ) ) {
				$status = get_post_meta( $order->get_id(), '_easypack_status',
					true );

				$tracking_url = false;
				if ( $status != 'new' ) {
					$tracking_url = EasyPack_Helper()->get_tracking_url();
					$parcels      = get_post_meta( $order->get_id(),
						'_easypack_parcels',
						true );
					if ( $parcels != '' ) {
						foreach ( $parcels as $parcel ) {
							$tracking_url .= $parcel['easypack_data']['id']
							                 . ',';
						}
					}
					$tracking_url = trim( $tracking_url, ',' );
				}

				if ( $tracking_url ) {
					$actions['easypack_tracking'] = [
						'url'  => $tracking_url,
						'name' => __( 'Track shipment',
							EasyPack::$text_domain ),
					];
				}

				$actions['fast_return'] = [
					'url'  => get_option( 'easypack_fast_return' ),
					'name' => __( 'Szybkie zwroty', EasyPack::$text_domain ),
				];
			}

			return $actions;
		}

		function woocommerce_email_after_order_table(
			$order,
			$is_admin,
			$plain_text
		) {

			if ( $order->has_shipping_method( $this->id ) ) {
				$status = get_post_meta( $order->get_id(), '_easypack_status',
					true );

				if ( $status != '' && $status != 'new' ) {
					$tracking_url    = false;
					$package_numbers = '';
					if ( $status != 'new' ) {
						$tracking_url = EasyPack_Helper()->get_tracking_url();
						$parcels      = get_post_meta( $order->get_id(),
							'_easypack_parcels', true );
						if ( ! is_array( $parcels ) ) {
							return;
						}
						foreach ( $parcels as $parcel ) {
							$tracking_url .= $parcel['easypack_data']['id']
							                 . ',';
							$package_numbers
							              = $parcel['easypack_data']['id']
							                . ', ';
						}
						$package_numbers = trim( trim( $package_numbers ),
							',' );
						$tracking_url    = trim( $tracking_url, ',' );
					}
					if ( $tracking_url ) {
						$args['tracking_url']    = $tracking_url;
						$args['package_numbers'] = $package_numbers;
						$args['logo']
						                         = untrailingslashit( EasyPack()->getPluginUrl() )
						                           . 'logo/small/white.png';
						if ( $plain_text ) {
							wc_get_template( 'emails/plain/after-order-table.php',
								$args, '',
								plugin_dir_path( EasyPack()->getPluginFilePath() )
								. 'templates/' );
						} else {
							wc_get_template( 'emails/after-order-table.php',
								$args, '',
								plugin_dir_path( EasyPack()->getPluginFilePath() )
								. 'templates/' );
						}
					}
				}
			}

		}

		/**
		 * @return ShipX_Shipment_Service
		 */
		public function getShipmentService() {
			return $this->shipment_service;
		}

		/**
		 * @param ShipX_Shipment_Service $shipment_service
		 */
		public function setShipmentService( $shipment_service ) {
			$this->shipment_service = $shipment_service;
		}

		/**
		 * @return bool
		 */
		protected function is_method_courier() {
			return $this->id === 'easypack_shipping_courier'
			       || $this->id === 'easypack_cod_shipping_courier'
			       || $this->id === 'easypack_shipping_courier_c2c'
			       || $this->id === 'easypack_shipping_courier_lse'
			       || $this->id === 'easypack_shipping_courier_local_standard'
			       || $this->id === 'easypack_shipping_courier_local_express'
			       || $this->id === 'easypack_shipping_courier_palette'
			       || $this->id === 'easypack_shipping_courier_lse_cod'
			       || $this->id
			          === 'easypack_shipping_courier_local_standard_cod'
			       || $this->id
			          === 'easypack_shipping_courier_local_express_cod'
			       || $this->id === 'easypack_shipping_courier_palette_cod';
		}
	}
}
